package com.huimv.business.farm.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huimv.business.farm.service.IFarmDevice;
import com.huimv.business.server.dao.entity.FarmDeviceEntity;
import com.huimv.business.server.dao.repo.FarmDeviceRepo;
import com.huimv.business.utils.DateUtil;
import com.huimv.business.utils.HttpTemplete;
import com.huimv.business.utils.SnowflakeIdWorker;
import com.huimv.business.utils.TextUtil;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import java.io.UnsupportedEncodingException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class FarmDeviceImpl implements IFarmDevice {
    @Autowired
    private FarmDeviceRepo deviceRepo;
    @Autowired
    private TextUtil textUtil;
    @Autowired
    private HttpTemplete httpTemplete;
    @Autowired
    private DateUtil dateUtil;
    @Value("${dataCenter.ipAddr}")
    private String DATA_CENTER_IP_ADDR;
    @Value("${dataCenter.device.port}")
    private String PORT;
    @Value("${dataCenter.device.sync}")
    private boolean dataSync;
    @Value("${dataCenter.device.addService}")
    private String deviceAddService;
    @Value("${dataCenter.device.editService}")
    private String deviceEditService;
    @Value("${dataCenter.device.removeService}")
    private String deviceRemoveService;
    @Value("${local.farmID}")
    private Integer FARM_ID;

    /**
     * @Method      : newDevice
     * @Description : 新建设备信息
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/17       
     * @Time        : 11:00
     */
    @Override
    public Result newDevice(String deviceData) throws UnsupportedEncodingException {
        //
        SnowflakeIdWorker idWorker = new SnowflakeIdWorker(1, 0);
        JSONObject deviceDataJo = JSON.parseObject(deviceData);
        FarmDeviceEntity deviceEntity = new FarmDeviceEntity();
        deviceEntity.setDeviceCode(deviceDataJo.getString("deviceCode"));
        deviceEntity.setDeviceName(deviceDataJo.getString("deviceName"));
        deviceEntity.setDeviceType(deviceDataJo.getString("deviceType"));
        deviceEntity.setFactory(deviceDataJo.getString("factory"));
        deviceEntity.setFarmId(FARM_ID);
        deviceEntity.setMainParams(deviceDataJo.getString("mainParams"));
        deviceEntity.setRecord(deviceDataJo.getString("record"));
        deviceEntity.setWorker(deviceDataJo.getString("worker"));
        deviceEntity.setState(deviceDataJo.getInteger("state"));
        deviceEntity.setLastTime(new Timestamp(new Date().getTime()));
        deviceEntity.setUpTime(new Timestamp(new Date().getTime()));
        deviceEntity.setLockinState(1);
        deviceEntity.setDataId(idWorker.newId());
        FarmDeviceEntity addDeviceEntity = deviceRepo.saveAndFlush(deviceEntity);
        log.info("牧场端添加设备信息>>"+addDeviceEntity);
        //同步更新数据中心端设备数据
        if(dataSync){
            log.info("同步数据中心端设备数据。");
            String serviceUrl = DATA_CENTER_IP_ADDR + PORT + deviceAddService;
            log.info("serviceUrl>>"+serviceUrl);
            String data = textUtil.encode(JSON.toJSONString(addDeviceEntity));
            log.info("base64密文>>"+data);
            //提交请求
            httpTemplete.doPostSimple(serviceUrl,data);
        }
        return new Result(ResultCode.SUCCESS, addDeviceEntity);
    }

    /**
     * @Method      : editDevice
     * @Description : 编辑设备信息
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/17       
     * @Time        : 11:00
     */
    @Override
    public Result editDevice(String deviceData) throws UnsupportedEncodingException {
        //
        SnowflakeIdWorker idWorker = new SnowflakeIdWorker(1, 0);
        JSONObject deviceDataJo = JSON.parseObject(deviceData);
        //查询记录是否存在
        Optional<FarmDeviceEntity> optional = deviceRepo.findById(deviceDataJo.getInteger("id"));
        if(!optional.isPresent()){
            String ERR_INFO_NOEXIST = "该记录不存在.";
            log.error(ERR_INFO_NOEXIST);
            return new Result(10001,ERR_INFO_NOEXIST,false);
        }
        //编辑记录
        FarmDeviceEntity deviceEntity =  optional.get();
        deviceEntity.setDeviceCode(deviceDataJo.getString("deviceCode"));
        deviceEntity.setDeviceName(deviceDataJo.getString("deviceName"));
        deviceEntity.setDeviceType(deviceDataJo.getString("deviceType"));
        deviceEntity.setFactory(deviceDataJo.getString("factory"));
        deviceEntity.setMainParams(deviceDataJo.getString("mainParams"));
        deviceEntity.setRecord(deviceDataJo.getString("record"));
        deviceEntity.setWorker(deviceDataJo.getString("worker"));
        deviceEntity.setState(deviceDataJo.getInteger("state"));
        deviceEntity.setLastTime(new Timestamp(new Date().getTime()));
        deviceEntity.setUpTime(new Timestamp(new Date().getTime()));
        deviceEntity.setLockinState(1);
        FarmDeviceEntity addDeviceEntity = deviceRepo.saveAndFlush(deviceEntity);
        log.info("牧场端添加设备信息>>"+addDeviceEntity);
        //同步更新数据中心端设备数据
        if(dataSync){
            log.info("同步数据中心端设备数据。");
            String serviceUrl = DATA_CENTER_IP_ADDR + PORT + deviceEditService;
            log.info("serviceUrl>>"+serviceUrl);
            String data = textUtil.encode(JSON.toJSONString(addDeviceEntity));
            log.info("base64密文>>"+data);
            //提交请求
            httpTemplete.doPostSimple(serviceUrl,data);
        }
        return new Result(ResultCode.SUCCESS, addDeviceEntity);
    }

    /**
     * @Method      : removeDevice
     * @Description : 删除设备信息
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/17       
     * @Time        : 11:00
     */
    @Override
    public Result removeDevice(String deviceData) throws UnsupportedEncodingException {
        JSONObject deviceDataJo = JSON.parseObject(deviceData);
        Integer id = deviceDataJo.getInteger("id");
        //查询记录是否存在
        Optional<FarmDeviceEntity> optional = deviceRepo.findById(id);
        if(!optional.isPresent()){
            String ERR_INFO_NOEXIST = "该记录不存在.";
            log.error(ERR_INFO_NOEXIST);
            return new Result(10001,ERR_INFO_NOEXIST,false);
        }
        Long dataId = optional.get().getDataId();
        //
        deviceRepo.deleteById(id);
        //同步牧场端设备数据
        if(dataSync){
            log.info("同步数据中心端设备数据。");
            String serviceUrl = DATA_CENTER_IP_ADDR + PORT + deviceRemoveService;
            log.info("serviceUrl>>"+serviceUrl);
            String data = textUtil.encode(JSON.toJSONString(dataId));
            log.info("base64密文>>"+data);
            //提交请求
            httpTemplete.doPostSimple(serviceUrl,data);
        }
        return new Result(ResultCode.SUCCESS);
    }

    /**
     * @Method      : listDevice
     * @Description : 设备信息列表展示
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/17       
     * @Time        : 9:22
     */
    @Override
    public Result listDevice(String deviceData){
        JSONObject deviceDataJo = JSON.parseObject(deviceData);
        String deviceName= deviceDataJo.getString("deviceName");
        String deviceCode = deviceDataJo.getString("deviceCode");
        String deviceType=deviceDataJo.getString("deviceType");
        String factory = deviceDataJo.getString("factory");
        String worker = deviceDataJo.getString("worker");
        Integer state = deviceDataJo.getInteger("state");
        Integer farmId = deviceDataJo.getInteger("farmId");
        String startDateText =  deviceDataJo.getString("startDate");
        String endDateText =  deviceDataJo.getString("endDate");
        Integer pageNo = deviceDataJo.getInteger("pageNo");
        Integer pageSize = deviceDataJo.getInteger("pageSize");

        Specification<FarmDeviceEntity> sf = (Specification<FarmDeviceEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != farmId) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(Integer.class), farmId));
            }
            if (null != deviceName) {
                predList.add(criteriaBuilder.equal(root.get("deviceName").as(String.class), deviceName));
            }
            if (null != deviceCode) {
                predList.add(criteriaBuilder.equal(root.get("deviceCode").as(String.class), deviceCode));
            }
            if (null != factory) {
                predList.add(criteriaBuilder.equal(root.get("factory").as(String.class), factory));
            }
            if (null != worker) {
                predList.add(criteriaBuilder.equal(root.get("worker").as(String.class), worker));
            }
            if (null != startDateText) {
                Date startDate = dateUtil.parseDate(startDateText);
                predList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("lastTime").as(Date.class), startDate));
            }
            if (null != endDateText) {
                Date endDate = dateUtil.parseDatePlus(endDateText);
                predList.add(criteriaBuilder.lessThanOrEqualTo(root.get("lastTime").as(Date.class), endDate));
            }
            if (null != state) {
                predList.add(criteriaBuilder.equal(root.get("state").as(Integer.class), state));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, deviceRepo.findAll(sf, pageable));
    }
}
